package com.gct.common.core.result;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

/**
 * @author: NewMeanning
 * @create: 2020-09-25 11:36
 **/
@ApiModel(value = "结果集", description = "接口常规返回结果集封装")
@Data
public class Result<T> {

    @ApiModelProperty(value = "状态码")
    private Integer status;

    @ApiModelProperty(value = "提示信息")
    private String message;

    @ApiModelProperty(value = "数据")
    private T data;

    @ApiModelProperty(value = "返回的数据类型")
    private Class<? extends Object> dataType;

    public Result(Integer status, String message, T data) {
        this.status = status;
        this.message = message;
        this.data = data;
        this.dataType = data != null ? data.getClass() : Object.class;
    }

    public Result(Integer status, String message, T data, Class<? extends Object> dataType) {
        this.status = status;
        this.message = message;
        this.data = data;
        this.dataType = dataType;
    }

    /**
     * @param metaType
     * @param <L>
     * @return
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @author lloyd
     * @since 2021.1.28
     */
    public <L> L toFiguredOutObject(Class<L> metaType) throws IllegalAccessException, InstantiationException, InvocationTargetException, NoSuchMethodException {
        if (data == null) return null;
        L m = metaType.newInstance();
        autoFillFields(m, data);
        return m;
    }

    /**
     * <p>
     * ps:
     * 1.请使用fastjson 1.2.58
     * 2.高维数组获取有问题
     * 3.基础类型 包括string ，非自建entity类型请酌情使用 或许有问题
     * </p>
     *
     * @param metaType
     * @param <L>
     * @return
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @author lloyd
     * @since 2021.1.28
     */
    public <L> List<L> toFiguredOutObjectList(Class<L> metaType) throws IllegalAccessException, InstantiationException, InvocationTargetException, NoSuchMethodException {
        if (data == null) return null;
        List r = null;
        if (data instanceof Page) {
            Page d = (Page) data;
            r = d.getRecords();
        } else if (data instanceof List)
            r = (List) data;
        else if (data instanceof JSONArray)
            r = (JSONArray) data;
        if (r == null || r.isEmpty())
            return null;
        List<L> rs = new ArrayList<L>();
        for (int i = 0; i < r.size(); i++) {
            L m = metaType.newInstance();
            autoFillFields(m, r.get(i));
            rs.add(m);
        }
        return rs;
    }

    /**
     * @param o1
     * @param o2
     * @throws IllegalAccessException
     * @author lloyd
     * @since 2021.1.28
     */
    private void autoFillFields(Object o1, Object o2) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
//        var fields = o1.getClass().getFields();//public fields
        Field[] fields = o1.getClass().getDeclaredFields();// all fields of public or private
        if (o2 instanceof JSONObject) {
            for (Field f1 : fields) {
                TableField an = f1.getDeclaredAnnotation(TableField.class);

                if (an != null && ((JSONObject) o2).containsKey(an.value())) {
                    setObjectField(f1, o1, ((JSONObject) o2).get(an.value()));
                    continue;
                }
                if (((JSONObject) o2).containsKey(f1.getName())) {
                    setObjectField(f1, o1, ((JSONObject) o2).get(f1.getName()));
                    continue;
                }
            }
        }
       /* var llifelds = o2.getClass().getFields();
        for (Field f1 : fields)
            for (Field f2 : llifelds) {
                if (f1.getName().equals(f2.getName()) || f2.getName().equals(f1.getDeclaredAnnotation(TableField.class).value())) {
                    f1.set(o1, f2.get(o2));
                    break;
                }
            }*/
    }

    private void setObjectField(Field f1, Object o1, Object value) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        if (f1 != null && value != null)
            if (f1.getType() != value.getClass())
                value = JSONObject.parseObject(value.toString(), f1.getType());

        if (Modifier.isPublic(f1.getModifiers())) {
            f1.set(o1, value);
        } else {
            String name = f1.getName();
            String firstChar = name.substring(0, 1).toUpperCase();
            if (name.length() > 1) name = firstChar + name.substring(1);
            else name = firstChar;
            name = "set" + name;
            Method method = o1.getClass().getMethod(name, f1.getType());
            if (method != null) {
                boolean accessible = method.isAccessible();
                method.setAccessible(true);
                method.invoke(o1, value);
                method.setAccessible(accessible);
            }

        }
    }

    public static <T> Result<T> create(Integer code, String msg, T data) {
        return new Result<>(code, msg, data);
    }

    public static <T> Result<T> create(CodeMsgInterface codeMsg, T data) {
        return new Result<>(codeMsg.getCode(), codeMsg.getMsg(), data);
    }

    public static <T> Result<T> create(CodeMsgInterface codeMsg) {
        return new Result<>(codeMsg.getCode(), codeMsg.getMsg(), null);
    }

    public static <T> Result ok(T data) {
        return Result.create(CodeMsgEnum.SUCCESS, data);
    }

    public static Result error(String message) {
        return Result.create(CodeMsg.createCodeMsg(-1, message));
    }

    @Override
    public String toString() {
        return "Result{" +
                "status=" + status +
                ", message='" + message + '\'' +
                ", data=" + data +
                '}';
    }

}